/*
    Aufgabe 2) Zweidimensionale Arrays und Methoden - Vier Gewinnt
*/

import java.awt.*;

public class Aufgabe2 {

    private static int[][] genGameBoard(int x, int y) {
        return new int[x][y];
    }

    private static void drawGameBoard(int[][] currentGameBoard, int oneSquareSize) {
        for (int i = 0; i < currentGameBoard.length; i++) {
            for (int j = 0; j < currentGameBoard[i].length; j++) {
                StdDraw.setPenColor(Color.BLUE);
                double centerX = i * oneSquareSize + oneSquareSize / 2.d;
                double centerY = j * oneSquareSize + oneSquareSize / 2.d;
                StdDraw.filledRectangle(centerX, centerY, oneSquareSize / 2.d, oneSquareSize / 2.d);
                switch (currentGameBoard[i][j]) {
                    case 0:
                        StdDraw.setPenColor(Color.GRAY);
                        break;
                    case 1:
                        StdDraw.setPenColor(Color.RED);
                        break;
                    case 2:
                        StdDraw.setPenColor(Color.YELLOW);
                        break;
                }

                StdDraw.filledCircle(centerX, centerY, oneSquareSize / 3.d);
            }
        }
    }

    private static boolean move(int[][] currentGameBoard, int player, int col) {
        for (int j = 0; j < currentGameBoard[col].length; j++) {
            if (currentGameBoard[col][j] == 0) {
                currentGameBoard[col][j] = player;
                return true;
            }
        }
        return false;
    }

    private static boolean checkGameStatus(int[][] currentGameBoard, int player) {
        for (int i = 0; i < currentGameBoard.length; i++) {
            for (int j = 0; j < currentGameBoard[i].length; j++) {
                if (currentGameBoard[i][j] == player) {
                    // check vertically
                    for (int verCount = 1; j + verCount <= currentGameBoard[i].length-1;) {
                        if (currentGameBoard[i][j+verCount]==player) verCount++; else break;
                        if (verCount>=4) return true;
                    }
                    // check horizontally
                    for (int horCount = 1; i + horCount <= currentGameBoard.length-1;) {
                        if (currentGameBoard[i+horCount][j]==player) horCount++; else break;
                        if (horCount>=4) return true;
                    }
                    // check diag1
                    for (int diag1Count = 1; i + diag1Count <= currentGameBoard.length-1 &&
                            j + diag1Count <= currentGameBoard[i].length-1;) {
                        if (currentGameBoard[i+diag1Count][j+diag1Count]==player) diag1Count++; else break;
                        if (diag1Count>=4) return true;
                    }
                    // check diag2
                    for (int diag2Count = 1; i - diag2Count >=0 &&
                            j + diag2Count <= currentGameBoard[i].length-1;) {
                        if (currentGameBoard[i-diag2Count][j+diag2Count]==player) diag2Count++; else break;
                        if (diag2Count>=4) return true;
                    }
                }
            }
        }
        return false;
    }

    public static void main(String[] args) {

        // canvas settings
        int rowsGameBoard = 6;
        int colsGameBoard = 7;
        int oneSquareSize = 50;
        int width = oneSquareSize * colsGameBoard;
        int height = oneSquareSize * rowsGameBoard;

        StdDraw.setCanvasSize(width, height);
        StdDraw.setXscale(0, width);
        StdDraw.setYscale(0, height);

        // game variables
        int[][] myGameBoard = genGameBoard(colsGameBoard, rowsGameBoard);
        boolean fullBoard = false;
        boolean isWon = false;
        int player = 1;
        int fieldsUsed = 0;

        // initial draw of the game board
        drawGameBoard(myGameBoard, oneSquareSize);

        StdDraw.setFont(new Font("Arial", Font.BOLD, 30));
        StdDraw.enableDoubleBuffering();

        // game play starts
        System.out.println("Player " + player + (player == 1 ? " (RED)" : " (YELLOW)") + " has to make a move!");
        while (!fullBoard && !isWon) {
            if (StdDraw.isMousePressed()) {
                int x = (int) StdDraw.mouseX();
                int col = x / oneSquareSize;

                if (move(myGameBoard, player, col)) {
                    fullBoard = ++fieldsUsed >= rowsGameBoard * colsGameBoard;
                } else {
                    StdDraw.setPenColor(StdDraw.PRINCETON_ORANGE);
                    StdDraw.text(oneSquareSize * colsGameBoard / 2.d, oneSquareSize * rowsGameBoard / 2.d,
                            "Column already full!");
                    StdDraw.show();
                    StdDraw.pause(200);
                    continue;
                }

                drawGameBoard(myGameBoard, oneSquareSize);

                if (fieldsUsed >= 7) isWon = checkGameStatus(myGameBoard, player);

                if (fullBoard) {
                    StdDraw.setPenColor(StdDraw.PRINCETON_ORANGE);
                    StdDraw.text(oneSquareSize * colsGameBoard / 2.d, oneSquareSize * rowsGameBoard / 2.d,
                            "Board full!!");
                }
                if (isWon) {
                    StdDraw.setPenColor(StdDraw.GREEN);
                    StdDraw.text(oneSquareSize * colsGameBoard / 2.d, oneSquareSize * rowsGameBoard / 2.d,
                            "Player " + player + (player == 1 ? " (RED)" : " (YELLOW)") + " won!");
                }

                StdDraw.show();
                StdDraw.pause(200);
                player = player == 1 ? 2 : 1;
                System.out.println("Player " + player + (player == 1 ? " (RED)" : " (YELLOW)") + " has to make a move!");
            }
        }
    }
}
